#ifndef POWERSAVE_ACPI_H
#define POWERSAVE_ACPI_H

#define ACPI_SLEEPFILE "/proc/acpi/sleep"


class ACPI_Interface : public PM_Interface
{
 private:
	
	// workaround for buggy double button events
	// only allow one button event(sleep/power)
	// inside the time window below (in millisecs)
	unsigned long last_button_occurence;
	const static unsigned int button_time_window = 1000;

	/** @brief sets the battery alarm of the current scheme
	 *
	 * @return integer value with the result of set batteryalarm
	 * @retval - 1 on success
         * @retval -1 if alarm could not be set or is not supported
	 */
	int setBatteryAlarm_();

	static string acpidSocketFile;
 public:
	// this is now configurable via -f parameter
	// it could be either the acpid socket or the kernel acpi event file
	static string ACPI_EVENTFILE;
	// default initialisation
	ACPI_Interface(GeneralConfig *config_obj);
        virtual ~ACPI_Interface();
	virtual int openHWEventFD();

	/** @brief Handles an event on the socket
	 *
	 * @return integer value with result if handle event
	 * @retval 1 if an event has been generated
	 * @retval 0 if no event has been generated
	 * @retval -1 on fatal errors
	 * @retval -2 if acpid shuts down/ socket is missing
	 */
	virtual int handleHWEventRequest();
	virtual int suspend_to_ram();
	virtual int standby();

	/** @brief get time in millisecs since 1970 */
	unsigned long getTimestamp();
	virtual int openAcpidSocket();
	virtual void activateSettings();
};

#endif
