/***************************************************************************
 *
 * brightness.cpp - Brightness Support for powersave daemon
 *
 * Copyright (C) 2005 Timo Hoenig <thoenig@suse.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 **************************************************************************/

#ifndef POWERSAVE_BRIGHTNESS_H
#define POWERSAVE_BRIGHTNESS_H

#define ACPI_ASUS    "/proc/acpi/asus/brn"
#define ACPI_IBM     "/proc/acpi/ibm/brightness"
#define ACPI_SONY    "/proc/acpi/sony/brightness"
#define ACPI_TOSHIBA "/proc/acpi/toshiba/lcd"

/* merge #include "powersaved.h" */

class Brightness {
    protected:
        const char* iface;
        int         fd;

    public:
	virtual ~Brightness();
	static Brightness *Probe();

        virtual void Init();

        virtual int  Get ();
        virtual void Set (int);

        virtual void Min ();
        virtual void Med ();
        virtual void Max ();

        virtual void Up ();
        virtual void Down ();

        virtual int  GetLevels ();
};

class BrightnessASUS : public Brightness {
    public:
        void Init();

        int  Get ();
        void Set (int);

        void Min ();
        void Med ();
        void Max ();

        void Up ();
        void Down ();

        int  GetLevels ();
}; 

class BrightnessIBM : public Brightness {
    public:
        void Init();

        int  Get ();
        void Set (int);

        void Min ();
        void Med ();
        void Max ();

        void Up ();
        void Down ();

        int  GetLevels ();
}; 

class BrightnessSony : public Brightness {
    public:
        void Init();

        int  Get ();
        void Set (int);

        void Min ();
        void Med ();
        void Max ();

        void Up ();
        void Down ();

        int  GetLevels ();
};

class BrightnessToshiba : public Brightness {
    public:
        void Init ();

        int  Get ();
        void Set (int);

        void Min ();
        void Med ();
        void Max ();

        void Up ();
        void Down ();

        int  GetLevels ();
};

#endif /* POWERSAVE_BRIGHTNESS_H */
