#ifndef CLIENTCONNECTION_H
#define CLIENTCONNECTION_H

#include "powersave_clientsocket.h"

#include <list>

class CPUFreq_Interface;
struct PM_STATE;

/** @brief  Base class for administrate clients
 *
 * Base class for administrate clients which connect to a server socket
 * provided by powersaved. Everytime one of the above defined states changes, 
 * all client which requested this state will be notified. Additionally it is
 * also possible that acpi_events occurring on /proc/acpi/events will be 
 * forwarded to the clients instead of changes.
 *
 * The class clientStorage is a list which holds all the registered clients.
*/
class clientConnection {
public:
	clientConnection();
	~clientConnection();

	/** @brief check if we have a connected client with a specific capability
	 *
	 * @param capability the capability to check for
	 *
	 * @return integer value representing result of function call
	 * @retval 1 on success
	 * @retval 0 otherwise
	*/
	bool checkClientCapabilities( int capability );

	/** @brief get number of clients connected
	 *
	 * @return integer with number of clients connected to the daemon
	 *
	 * invokes count() from class clientStorage and so returns the
	 * number of clients which are connected to the daemon
	 */
	int count();

	/** @brief adds a client to the list of connected clients
	 *
	 * @param name the dbus name of the client
	 * @param modes the modes the client is aware of
	 *
	 * @return true on success, false otherwise
	 */
	bool addClient( const char *name, int modes );


	/** @brief check if given client should get removed 
	 *
	 * @param name the dbus name of the client
	 *
	 * @return true when the client got removed, false otherwise
	 */
	bool checkClientRemoval( const char* name );

private:
	/** list for storing all connected clients */
	list<cNode> clients;

};

#endif
