
#include <stdlib.h>
#include <powerlib.h>
#include "dbus_server.h"
#include <stddef.h>
#include "powersaved.h"


int DBus_Server::DISPATCH_COMPLETE =         1;
DBusWatch *DBus_Server::active_watch =       NULL;
DBusConnection *DBus_Server::conn =          NULL;
// We better define it here, so DBusDied can be invoked from DBus process
// context and we can reinit the connection one select cycle faster
bool DBus_Server::DBusDied = false;

#define NO_MEM "No memory\n"

static dbus_bool_t
add_watch (DBusWatch *watch,
	   void   *data)
{
        pDebug (DBG_DEBUG, "");
	
	// not needed for now
	// DBusConnection *conn = (DBusConnection*) data;
	//	uint32_t flags;
	//	int watch_fd;
	
	if (!dbus_watch_get_enabled (watch)){
		pDebug (DBG_DIAG, "Cannot enable watch");
		return TRUE; // TODO: why do we return true here? we failed!
                             //       OTOH we are always called twice, and the first invocation always fails...
	}
	
	else{
		DBus_Server::active_watch = watch;

		pDebug (DBG_INFO, "Watch is enabled - fd: %d",
		       dbus_watch_get_fd(DBus_Server::active_watch));
		/*
		flags = dbus_watch_get_flags(watch);

		if (flags & DBUS_WATCH_READABLE){
			DBus_Server::readable_fds[watch_fd] = watch;
			pDebug (DBG_INFO, "fd %d added to readable dbus fds", 
				watch_fd);
		}
		if (flags & DBUS_WATCH_WRITABLE){
			DBus_Server::writeable_fds[watch_fd] = watch;
			pDebug (DBG_INFO, "fd %d added to writeable dbus fds", 
				watch_fd);
		}
		*/
	}
	//	dbus_watch_set_data(watch,  NULL, NULL);
	return TRUE;
}

static void
remove_watch (DBusWatch *watch,
	      void *data)
{
        pDebug (DBG_INFO, "");

	if (!dbus_watch_get_enabled (watch)){
		pDebug (DBG_DIAG, "Cannot enable watch.");

	}
#ifdef NOTDEF
	else{
		watch_fd = dbus_watch_get_fd (watch);
		if (watch_fd < 0){
			pDebug (DBG_INFO, "Watch is not enabled");
			return;
		}
		pDebug (DBG_INFO, "Watch is enabled - fd: %d",
			watch_fd);
		flags = dbus_watch_get_flags(watch);

		if (flags & DBUS_WATCH_READABLE){
			DBus_Server::readable_fds.erase(watch_fd);
			pDebug (DBG_INFO, "fd %d added to readable dbus fds", 
				watch_fd);
		}
		if (flags & DBUS_WATCH_WRITABLE){
			DBus_Server::writeable_fds.erase(watch_fd);
			pDebug (DBG_INFO, "fd %d added to writeable dbus fds", 
				watch_fd);
		}
	}
#endif
}

static void
watch_toggled (DBusWatch *watch,
               void      *data)
{
        pDebug (DBG_INFO, "");
	/* Remove fds?!? */
}

static dbus_bool_t
add_timeout (DBusTimeout *timeout,
	   void   *data)
{
	/*	if (watch != NULL){
		pDebug (DBG_ERR, "Watch fd: %d", watch->fd);
	*/
        pDebug (DBG_DEBUG, "");
	if (!dbus_timeout_get_enabled (timeout))
		return FALSE;

	dbus_timeout_set_data (timeout, DBus_Server::conn, NULL);

	return TRUE;
}

static void
remove_timeout (DBusTimeout *timeout,
	      void *data)
{
        pDebug (DBG_DEBUG, "");
}

static void
timeout_toggled (DBusTimeout *timeout,
               void      *data)
{
  /* Because we just exit on OOM, enable/disable is
   * no different from add/remove
   */
        pDebug (DBG_INFO, "");
}

static void
connection_setup_free(void *data)
{
        pDebug (DBG_INFO, "");
}


/** @brief DBus callback function, don't know exactly
           what else could be done with it
*/
static void
DBusDispatchStatusFunc(struct DBusConnection *connection,
			   DBusDispatchStatus new_status,
			   void           *data){
	pDebug (DBG_INFO, "");	

	switch (new_status){
	case DBUS_DISPATCH_DATA_REMAINS:
		DBus_Server::DISPATCH_COMPLETE = 0;
		pDebug(DBG_INFO, "We have data to process");
		break;
	case DBUS_DISPATCH_COMPLETE:
		DBus_Server::DISPATCH_COMPLETE = 1;
		pDebug(DBG_INFO, "Data has been processed");
		break;
	case DBUS_DISPATCH_NEED_MEMORY:
		pDebug(DBG_INFO, "We need more memory");
		break;
	}
}


/*
  -----------------------------
  class DBus_Server begins here
  -----------------------------
*/


/** Maybe we need to store several file descriptors (watches) later,
    until now I only got one per connection

map<int,DBusWatch*> DBus_Server::readable_fds = map<int,DBusWatch*>();
map<int,DBusWatch*> DBus_Server::writeable_fds = map<int,DBusWatch*>();
*/


//int DBus_Server::updateFds(fd_set &readable, fd_set &writeable){
/* not needed if we only have to care about one fd
int DBus_Server::updateFds(fd_set &readable){

	pDebug (DBG_INFO, "");
	for (map<int,DBusWatch*>::iterator it = readable_fds.begin();
	     it != readable_fds.end(); it++){
		pDebug(DBG_DIAG, "DBus added fd: %d", it->first);
		//		if (FD_ISSET(*it, readable)
		FD_SET(it->first, &readable);
	}
	return 1;
}
*/


DBus_Server::DBus_Server(){
	/* make this static ? */
	conn = NULL;
}

DBus_Server::~DBus_Server(){

	/* Destroy connection properly */
	if (conn != NULL)
		dbus_connection_disconnect(conn);
}


DBusHandlerResult
DBus_Server::filter_function(DBusConnection *conn, DBusMessage *msg, void *data){

        pDebug (DBG_DEBUG, "");	
	
	/* Be careful, handle error at once and go out, or other functions
	   may fail ungracefully ... */

	if (msg == NULL){
		return DBUS_HANDLER_RESULT_HANDLED;
		pDebug (DBG_ERR, "Invalid message received");
	}

	/** @ToDo This means DBus daemon itself threw us out,
	           Need to disconnect gracefully and free all resources
	*/
	if (dbus_message_is_signal (msg,
				    DBUS_INTERFACE_LOCAL,
				    "Disconnected")){
		pDebug (DBG_ERR, "We got a disconnected signal");
		// only way I see for now to get this information
		DBusDied = true;
		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}

	int type = dbus_message_get_type (msg);		
	if (type == DBUS_MESSAGE_TYPE_ERROR){
		pDebug (DBG_ERR, "Received error message: %s", 
			dbus_message_get_error_name(msg));
		return DBUS_HANDLER_RESULT_HANDLED;
	}
	/* Have to use const char* instead of string here.
	 * Return value might be NULL
	 */
	const char* t = dbus_message_get_interface(msg);
	if (t == NULL){
		pDebug (DBG_INFO, "Received message from invalid interface");
		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}
	const string interface = t;
	/* We only handle messages which want a reply */
	int reply_wanted = !dbus_message_get_no_reply(msg);

	// We do not compare the object path, I see no need for ...
	//	    && strcmp(dbus_message_get_path(msg), DBUS_MESSAGE_BUS_PATH) == 0) {
	
	/* ToDo: what should be filtered here and what should be filtered
	         in path_message_func (for what is this vtable stuff good for) ?!?
		 -> Deleted the vtable path_message_func stuff for now, all
		    message processing is started here
	*/

	DBusHandlerResult ret = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	
	int reply_error_id;

	DBusMessage *reply;
	/** @brief checking of interfaces is done here in filter function
	 *         checking for the right member, message contents, etc. must
	 *         be done in pm->handleDBusRequest(..)
	 *         Create the Message here, we must send a reply (sendReply(),
	 *         there the memory for the reply will be freed again.
	 */
	if (interface == PS_DBUS_MANAGER_INTERFACE){
		reply = dbus_message_new_method_return(msg);
		if (type != DBUS_MESSAGE_TYPE_METHOD_CALL || !reply_wanted){
			pDebug (DBG_ERR, "Invalid msg type (not method_call) sent to manager interface");
			sendReply(reply, REPLY_INVALID_MESSAGE_TYPE);
		}
		else{
			pDebug (DBG_DIAG, "Method sent to manager interface");
			reply_error_id = pm->handleDBusRequest(msg, reply, MANAGER_MESSAGE);
			sendReply(reply, reply_error_id);
		}
		ret = DBUS_HANDLER_RESULT_HANDLED;
	}
	else if (interface == PS_DBUS_REQUEST_INTERFACE){
		reply = dbus_message_new_method_return(msg);
		if (type != DBUS_MESSAGE_TYPE_METHOD_CALL || !reply_wanted){
			pDebug (DBG_ERR, "Invalid msg sent to request interface");
			sendReply(msg, REPLY_INVALID_METHOD);
		}
		else{
			pDebug (DBG_INFO, "Method sent to request interface");
			reply_error_id = pm->handleDBusRequest(msg, reply, REQUEST_MESSAGE);
			sendReply(reply, reply_error_id);
		}
		ret = DBUS_HANDLER_RESULT_HANDLED;
	}
	else if (interface == PS_DBUS_ACTION_INTERFACE){
		reply = dbus_message_new_method_return(msg);
		if (type != DBUS_MESSAGE_TYPE_METHOD_CALL || !reply_wanted){
			pDebug (DBG_ERR, "Invalid msg sent to action interface");
			sendReply(msg, REPLY_INVALID_METHOD);
		}
		else{
			pDebug (DBG_INFO, "Method sent to action interface");
			reply_error_id = pm->handleDBusRequest(msg, reply, ACTION_MESSAGE);
			sendReply(reply, reply_error_id);
		}
		ret = DBUS_HANDLER_RESULT_HANDLED;
	}
	else if (interface == DBUS_INTERFACE_DBUS){
		if ( type != DBUS_MESSAGE_TYPE_SIGNAL ){
			pDebug (DBG_ERR, "Invalid signal received");
			/** @ToDo HANDLED or NOT_YET_HANDLED, does it matter ?!? */
			ret = DBUS_HANDLER_RESULT_HANDLED;
		}
		else{
			pDebug( DBG_INFO, "Signal from %s with member %s received",
				DBUS_INTERFACE_DBUS, dbus_message_get_member(msg));
			
			string msg_member = dbus_message_get_member(msg);			
			if ( msg_member == "NameOwnerChanged"){
				
				char *old_owner;
				char *new_owner;
				/** @ToDo Error catching... */
				
				/* get the second (the old owner) and the third (the new owner)
				   string of the message (first string: affected owner) */
				if (!dbusGetMessageString(msg, &old_owner, 1)
				    && !dbusGetMessageString(msg, &new_owner, 2)
				    && old_owner != NULL
				    && new_owner != NULL){
					/* if the new owner is "" the client broke away -> remove
					   from client list */
					if (!strcmp (new_owner, ""))
						pm->cC->checkClientRemoval( old_owner );
				}
			}
		}
	}
	else {
		/* This message is noy for us -> we are the only
		 * server offering com.novell.powersave
		 * -> client probably made a mistake and specified a wrong
		 * interface -> at least warning
		 * We cannot answer to an interface we don't know (I expect).
		 */
		pDebug (DBG_WARN, "Received a message for a foreign interface");
		ret = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}
	pDebug (DBG_INFO, "Message handled");
	return ret;
}


/* This is our main function that has to be invoked
 * if filedescriptors (watches) have been activated.
 * What means that something is going on on one
 * of our interfaces
 */

int DBus_Server::handleRequest(){

        pDebug (DBG_DEBUG, "");	
	
	uint32_t flags = 0;
	
	/*
	for (map<int,DBusWatch*>::iterator it = readable_fds.begin();
	     it != readable_fds.end(); it++){
		watch = it->second;
		flags |= DBUS_WATCH_READABLE;
		pDebug (DBG_INFO, "fd %d is readable", it->first);
				break;
	}
	// @ToDo: don't care about writable watches at the moment
	if (watch == NULL){
		for (map<int,DBusWatch*>::iterator it = readable_fds.begin();
		     it != readable_fds.end(); it++){
			if (FD_SET(it->first, &readable)){
				watch = it->second;
				flags |= DBUS_WATCH_READABLE;
				pDebug (DBG_INFO, "fd %d is readable", it->first);
				break;
			}
		}
	}
	*/
	/* further checking needed !?!
	int fd_flags = fcntl (dbus_watch_get_fd(active_watch), F_GETFL);
	*/
	
	if (active_watch == NULL){
		// The connection probably broke away ...
		pDebug (DBG_ERR, "Invalid watch");
		return -1;
	}
	flags |= DBUS_WATCH_READABLE;
	dbus_watch_handle(active_watch, flags);
	
	/* Dispatch messages */
	while (dbus_connection_dispatch(conn) == DBUS_DISPATCH_DATA_REMAINS){
		pDebug (DBG_INFO, "Message dispatching");
	};

	return 0;
}

int DBus_Server::openSystemConnection(){
	
	DBusError error;
	dbus_error_init(&error);
	if (conn != NULL){
		if (dbus_connection_get_is_connected (conn)){
			dbus_connection_close (conn);
		}
			dbus_connection_unref (conn);
			conn = NULL;
	}

	//		dbus_connection_set_change_sigpipe(TRUE);
	// pDebug(DBG_DIAG, "Invoke dbus_get");
	//	conn = dbus_bus_get(DBUS_BUS_SYSTEM, &error);
	conn = dbus_connection_open_private ("unix:path=/var/run/dbus/system_bus_socket",
					     &error);


	if (dbus_error_is_set(&error)) {
		pDebug(DBG_ERR, "dbus_bus_get(): Error. (%s)", error.message);
		dbus_error_free( &error );
		return -1;
	}
	else {
		dbus_connection_set_exit_on_disconnect(conn, 0);
		pDebug(DBG_INFO,"dbus_bus_get(): Success.");
	}
	dbus_bus_register( conn, &error);

	dbus_bus_request_name(conn, PS_DBUS_SERVICE, 0,
			      &error);

	if (dbus_error_is_set(&error)) {
		pDebug(DBG_ERR,"dbus_bus_request_name(): Error. (%s)",
		       error.message);
		if (conn != NULL){
			dbus_connection_close (conn);
			dbus_connection_unref (conn);
		}
		dbus_error_free( &error );
		return -1;
	}
	dbus_int32_t connection_slot = -1;
	dbus_connection_allocate_data_slot (&connection_slot);
	if (connection_slot < 0)
		{pDebug(DBG_ERR, "No Memory");return -1;};
	
	if (!dbus_connection_set_data (conn, connection_slot, conn,
				       (DBusFreeFunction)connection_setup_free))
		{pDebug(DBG_ERR, "No Memory");return -1;};
	

	if (!dbus_connection_set_watch_functions (conn,
						  add_watch,
						  remove_watch,
						  watch_toggled,
						  conn, NULL))
		{pDebug(DBG_ERR, "No Memory");return -1;};
	if (!dbus_connection_set_timeout_functions (conn,
						    add_timeout,
						    remove_timeout,
						    timeout_toggled,
						    NULL, NULL))
		{pDebug(DBG_ERR, "No Memory");return -1;};


	/*
	dbus_connection_set_wakeup_main_function (conn,
						  DBus_Server::handle_request,
						  NULL, NULL);
	*/

	dbus_connection_set_dispatch_status_function (conn,
						      DBusDispatchStatusFunc,
						      NULL, NULL);


		
	dbus_connection_add_filter(conn, filter_function, NULL, NULL);

	dbus_bus_add_match(conn,
			   "type='method_call',"
			   "interface='com.novell.powersave.manager',"
			   "member='connect'",
			   NULL);

	/* keep track of disconnecting clients */
	dbus_bus_add_match(conn,
			   "type='signal',"
			   "interface='org.freedesktop.DBus',"
			   "member='NameOwnerChanged'",
			   NULL);

	/*
	
        if (!dbus_connection_register_object_path (conn,
						   BUS_PATH,
						   &echo_vtable,
						   (void*) 0xdeadbeef)){
		{pDebug(DBG_ERR, "No Memory");exit(255);};
	}
	
	{
		void *d;
		if (!dbus_connection_get_object_path_data (conn, BUS_PATH, &d))
			{pDebug(DBG_ERR, "No Memory");exit(255);};
		if (d != (void*) 0xdeadbeef)
			{pDebug(DBG_ERR, "No Memory");exit(255);};
	}
	*/
	return 0;
}


int DBus_Server::getFd(){

	if (active_watch)
		return dbus_watch_get_fd(active_watch);
	else
		return -1;
}

void DBus_Server::sendReply(DBusMessage *reply, 
			    int error_id){

	/* We do not generate DBus error messages ourselves
	   If needed, this is the place to do.
	   Problem: we need to generate the message here
	            and do not know whether the
		    message should become an error message
           Maybe we could convert the message to an error message
	   later, but we should not need that...

	dbus_message_set_error_name (DBusMessage  *message,
	                             const char   *error_name)
	reply = dbus_message_new_method_return(msg);
	*/
 
	if (reply == NULL){
		pDebug (DBG_WARN, "Invalid reply");
		return;
	}
		
	pDebug (DBG_INFO, "Send error_id %d, '%s'", error_id, DBus_Error_Array[error_id]);
	/* attache contents collected from raw_reply_msg to DBusMessage reply */

	
	/* We only use DBUS_TYPE_UINT16 once: to indicate the error code */
	dbus_message_append_args (reply, DBUS_TYPE_UINT16, &error_id,
				  // don't need to also send the error string ?
				  // or could it be useful dbus-monitor ?
				  // DBUS_TYPE_STRING, &DBus_Error_Array[error_id],
				  DBUS_TYPE_INVALID);

	// check for memory failure? -> then FALSE will be returned
	if (reply == NULL){
		pDebug (DBG_WARN, "Could not send reply/error messge");
		return;
	}
	dbus_connection_send (DBus_Server::conn, reply, NULL);
	dbus_connection_flush (DBus_Server::conn);
	dbus_message_unref(reply);
}

int DBus_Server::emitSignal( const char *signal, const char *value ) {

	DBusMessage *message;
	
	if ( conn == NULL ) {
		pDebug(DBG_INFO, "Connection to dbus not ready, skipping signal %s", value);
		return false;
	}

	message = dbus_message_new_signal( "/com/novell/powersave",
					   "com.novell.powersave.manager",
					   signal );

	/* append the event to the signal */
	dbus_message_append_args( message,
				  DBUS_TYPE_STRING, &value,
				  DBUS_TYPE_INVALID );

	/* and send the message */
	if ( ! dbus_connection_send( conn,
				     message,
				     NULL ) ) {
		pDebug (DBG_ERR, "Could not send message to D-BUS system bus.\n");
		return false;
	}


	
	dbus_message_unref( message );
	/* not closing connection. If this function is called from within
	 * the daemon, we would close the global connection to dbus
	 */
//	dbus_connection_close( connection );
//	dbus_connection_unref( connection );

	return true;

}
