#ifndef DBUS_SERVER_H
#define DBUS_SERVER_H
#define DBUS_API_SUBJECT_TO_CHANGE 1

#include <dbus-1.0/dbus/dbus.h>

#include <iostream>
#include <map>
#include <queue>

#include "powersave_dbus.h"

using namespace std;

/** 
 * @defgroup DBus Server class
 * @brief   The DBus Server class implements a DBus service
 */

class DBus_Server {

 private:
	
	static DBusHandlerResult filter_function(DBusConnection *conn, 
						 DBusMessage *msg, void *data);
	/** @brief returns the fd descriptor to put a poll/select on
	 *  and wait for dbus activity.
	 *  
	 *  @return negativ value on failure
	*/

	static void sendReply(DBusMessage *reply, int error_id);

 public:
	static bool DBusDied;
	static int DISPATCH_COMPLETE;
	
	static DBusWatch *active_watch;

	static DBusConnection *conn;

	DBus_Server();
	~DBus_Server();

	/* make watch callback functions static and these private
	static map<int,DBusWatch*> readable_fds;
	static map<int,DBusWatch*> writeable_fds;
	*/


	/**
	 * @brief Opens a new connection to the system bus,
	 *        tries to register the service and returns
	 *        a filedescriptor to watch for
	 *
	 * @return    filedescriptor to watch on or negative value on failure         
	 */

	//	static int open_system_connection(int (*messageHandler)(std::string));
	static int openSystemConnection();

	int getFd();

	
	/** @brief This handles a DBus request. It has to be invoked
	 *         if the DBus fd got readable/writeable.
	 *         This function equals the main loop, but is invoked after
	 *         we went out of select/poll.
	 *         After calling this you can use it for select/poll again.
	 *  @ToDo: Evaluate whether we need to support multiple fds/watches.
	 *  @return: returns whether the request could be handled successfully 
	 *  @retval  negative value on failure (probably broken away connection)
	 *  @retval  0 on success
	 */
	//	static int handleRequest(fd_set &readable, fd_set &writeable);
	static int handleRequest();

	/** @brief emits a signal over the system bus
	 *
	 * @param signal the signal to emit
	 * @param value the value of the signal
	 * 
	 * @return DBUS_ERROR
	*/
	static int emitSignal( const char *signal, const char *value );
};

#endif // POWERSAVE_DBUS_SERVER_H
