#include <string>
#include <signal.h>
#include <sys/wait.h>
#include <errno.h>
#include <iterator>
#include <sys/time.h>

#include "cpufreq.h"
#include "event.h"
#include "powersaved.h"

using namespace std;
using namespace __gnu_cxx;

/* Iterator for single linked list Event::programs_2_execute */
typedef slist<string>::const_iterator ExecIter;

string Event::scripts_dir = "/usr/lib/powersave/scripts/";

Event::Event(string n):name(n){
        id = -1;
	timeout = -1;
//	clientFD = -1;
	starttime = -1;
	sleep_time = 0;
	current_program_2_execute = programs_2_execute.begin();
	setActions2Execute("ignore");
};

Event::Event(){
	id = -1;
	timeout = -1;
//	clientFD = -1;
	starttime = -1;
	sleep_time = 0;
	name = "default";
	current_program_2_execute = programs_2_execute.begin();
}

Event::Event(const Event& e){
	id = e.id;
	starttime = e.starttime;
	sleep_time = e.sleep_time;
	timeout = e.timeout;
//	clientFD= e.clientFD;
	
	name = e.name;
	programs_2_execute = e.programs_2_execute;
	current_program_2_execute = programs_2_execute.begin();
}

Event& Event::operator=(Event& e){
	id = e.id;
	starttime = e.starttime;
	sleep_time = e.sleep_time;
	timeout = e.timeout;
//	clientFD= e.clientFD;
	
	name = e.name;
	programs_2_execute = e.programs_2_execute;
	current_program_2_execute = programs_2_execute.begin();
	return *this;
}

Event::~Event(){
	if (!programs_2_execute.empty())
		programs_2_execute.clear();
}

int Event::execute_program(PM_Interface *pm_object, string acpi_event_line) {
	pid_t pid;
	int x = 0;
	int eventID = ID();

	// set the program to execute to the current one
	string exec_script_path;

	/* This event has no actions, needed to not segfault if empty */
	 
	if (programs_2_execute.empty())
		return 1;

	ExecIter exec_prog = current_program_2_execute;

	if (isInternalAction(*exec_prog) > 0){
		if (executeInternalAction(pm_object, *exec_prog) < 0){
			pDebug(DBG_DIAG, "Internal action %s returned an error", (*exec_prog).c_str());
			string c = "internal action failed: ";
			c.append( *exec_prog );
			pm_object->checkScriptReturn(id, 1, c.c_str());
			return 0;
		}
		else {
			pDebug(DBG_INFO, "Internal action %s executed", (*exec_prog).c_str());
			string c = "internal action successfull: ";
			c.append( *exec_prog );
			pm_object->checkScriptReturn(id, 0, c.c_str());
		}
		return 1;
	}

	exec_script_path = scripts_dir + *exec_prog;
	if (access(exec_script_path.c_str(), X_OK)){
		pDebug(DBG_WARN, "Could not execute program: %s for event %s - %s\n"
		       , exec_script_path.c_str(), name.c_str(), strerror(errno));
		string c = "Cannot access or execute: ";
		c.append( *exec_prog );
		pm_object->checkScriptReturn(id, 1, c.c_str());
		return 0;
	}

	pid = fork();
	//child
	if (pid == 0){
		int argCount = 6;
		char *argv[ argCount ];
		
        signal(SIGHUP, SIG_DFL);
		signal(SIGTERM, SIG_DFL);
		signal(SIGINT, SIG_DFL);
		signal(SIGQUIT, SIG_DFL);
		signal(SIGPIPE, SIG_DFL);
		signal(SIGCHLD, SIG_DFL);
		// close all fds
		int fdlimit = sysconf(_SC_OPEN_MAX);
		nice(proxy_nice_factor);
		if (nice < 0){
			pDebug(DBG_WARN, "Could not set nice factor for proxy execution: %s\n", strerror(errno));
		}

		for (int fd = 0; fd < fdlimit; fd++)
				close(fd);

		for (x = 0; x < argCount; x++)
		argv[x] = NULL;
		x = 0;
		
		pDebug(DBG_INFO, "Going to execute external action %s", (*exec_prog).c_str());

		/* first give name of the program as parameter */
		argv[x] = strdup((*exec_prog).c_str());
		x++;
		
		/* second: give name of happening event */
		argv[x] = strdup(name.c_str());
		x++;
		
		/* third is the current scheme file name */
		argv[x] = strdup(pm_object->config->current_scheme->file_name.c_str());
		x++;
		
		/* forth is acpi line */
		if (acpi_event_line == ""){
			acpi_event_line = "empty";
		}
		argv[x] = strdup(acpi_event_line.c_str());
		x++;
		
		/* fifth is the event id */
		argv[x] = new char[5];
		sprintf(argv[x], "%d", eventID);
		x++;
	
		argv[x]  = NULL;
		
		pDebug(DBG_INFO, "Program %s will be executed for event %s!\n Param 1: %s,"
		       "Param 2: %s\n", exec_script_path.c_str(), name.c_str(), argv[1], argv[2]);
		
		execv(exec_script_path.c_str(), argv);
		pDebug(DBG_ERR, "Came back from subprocess when trying to execute %s for event %s; Error: %s ", 
		       exec_script_path.c_str(), name.c_str(), strerror(errno));
		_exit(127);
	}
	else if (pid == -1){	// fork failed
		pDebug(DBG_ERR, "Could not fork process: %s!", strerror(errno));
	}
	else {
		// parent
		// no need to wait for return of script because it returns
		// through the script_return_helper. So return to main loop and wait
		// for other events from acpi or sockets
	}
	return 1;
}

void Event::setActions2Execute(const string actions_line){

	string action;
	int x = actions_line.size();
	int y = 0;
	if (actions_line == "") 
		return;

	programs_2_execute.clear();

	while (x > 0 && isspace(actions_line[x-1]))
	       x--;
	y=x-1;
	do {
		while (y > 0 && !isspace(actions_line[y])){
			y--;
		}
		if (y != 0)
			y++;
		if (x > y){
			action = actions_line.substr(y, x-y);
			programs_2_execute.push_front(action);
		}
		else
			break;
		y--;
		while (y > 0 && isspace(actions_line[y]))
			y--;
		x = y+1;
	}while (y > 0);
}

/* use a kind of hash set to find internal actions faster, see 
   const static Event::internal_actions above 

   this is really ugly, do it better !!!
*/
int Event::isInternalAction(const string action){

	if (!action.compare("ignore") || !action.compare("throttle") || !action.compare("dethrottle") ||
	    !action.compare("do_suspend_to_disk") || !action.compare("do_suspend_to_ram") || !action.compare("do_standby") ||
	    !action.compare("suspend_to_disk") || !action.compare("suspend_to_ram") || !action.compare("standby") 
	    || !action.compare("screen_saver")
	    || !action.compare("reread_cpu_capabilities") )
		return 1;
	else
		return -1;
}


int Event::executeInternalAction(PM_Interface *pm_object, const string action){

	int ret = 1;

	pDebug(DBG_INFO, "Executing internal action: %s", action.c_str());

	if (!action.compare("ignore")){
		/* ... */
	}
	else if (!action.compare("throttle")){
		pm_object->throttle();
	}
	else if (!action.compare("dethrottle")){
		pm_object->dethrottle();
	}
	/* really do the suspend/standby */
	else if (!action.compare("do_suspend_to_disk")){
		pm_object->sleep_wrapper("suspend2disk");
		pm_object->executeScript( pm_object->SLEEP_SCRIPT, "suspend2disk");
		pm_object->sleep_wrapper("suspend2disk");
	}
	else if (!action.compare("do_suspend_to_ram")){
		pm_object->sleep_wrapper("suspend2ram");
		pm_object->suspend_to_ram();
		pm_object->sleep_wrapper("suspend2ram");
	}
	else if (!action.compare("do_standby")){
		pm_object->sleep_wrapper("standby");
		pm_object->standby();
		pm_object->sleep_wrapper("standby");
	}
	else if (!action.compare("screen_saver")){
		pm_object->screenlock(this->name);
	}
	/* throw a suspend/standby event (will trigger other needed
	   actions for proper suspend/standby */
	else if (!action.compare("suspend_to_disk")){
		pm_object->executeEvent("global.suspend2disk");
	}
	else if (!action.compare("suspend_to_ram")){
		pm_object->executeEvent("global.suspend2ram");
	}
	else if (!action.compare("standby")){
		pm_object->executeEvent("global.standby");
	}
	else if (!action.compare("reread_cpu_capabilities")){
		if (CPUFreq_Interface::isSupported()){
			CPUFreq_Interface::rereadFrequencies();
		}
	}
	else
		ret = -1;
	return ret;
}


/*
---------------------------------------------------------------------
   the following is needed for managing events
---------------------------------------------------------------------
*/

unsigned long Event::timeElapsed() {
	struct timeval cur_time;
	struct timezone dummy;
	gettimeofday( &cur_time, &dummy );

	// time elapsed in milliseconds
	unsigned long time_elapsed = cur_time.tv_sec * 1000 + cur_time.tv_usec / 1000 -
		starttime - sleep_time;
	
	return time_elapsed;
}

eventStatus Event::getState() {
	return state;
}

void Event::setTimeout(unsigned long m_seconds) {
	timeout = m_seconds;
}

unsigned long Event::getTimeout() {
	return timeout;
}

void Event::setID(int i) {
	id = i;
}

int Event::ID() {
	return id;
}

void Event::setState(eventStatus st) {
	state = st;
}

//int Event::getClientFD() {
//	return clientFD;
//}

slist<string>::iterator Event::end() {
	return programs_2_execute.end();
}

void Event::start() {
	struct timeval cur_time;
	struct timezone dummy;
	gettimeofday( &cur_time, &dummy );

	this->starttime = cur_time.tv_sec *1000 + cur_time.tv_usec / 1000;
}

void Event::increaseSleeptime(unsigned long time) {
	sleep_time += time;
}





/*
---------------------------------------------------------------------
   end of managing part
---------------------------------------------------------------------
*/



#ifdef CREATE_DEBUG_OUTPUT

ostream& operator<<(ostream& os, const Event& ev){

	 copy(ev.programs_2_execute.begin(), ev.programs_2_execute.end(),
	      ostream_iterator<string>(os));
	 return os;
}

#endif
