#include "event_management.h"
#include "event.h"

#include <string>

EventManagement::EventManagement(){
	id_count = 0;
}


EventManagement::~EventManagement(){}


int EventManagement::registerEvent(Event& event, time_t timeout) {
	
        pDebug(DBG_INFO, "registering event no.'%d'" , id_count);
	// give the event an unique id
	event.setID(id_count);
	// the next event will get a higher id
	id_count++;
	// set the timeout in milliseconds
	event.setTimeout(timeout*1000);
        // start the counter which looks if an event timed out
	event.start();
	// not used at the moment
	event.setState(EVENT_WAITING);

	// append the event to the list of pending events
	Events.push_back(event);

	return event.ID();
}


int EventManagement::remove(int id) {
	for (std::list<Event>::iterator it = Events.begin();
	     it != Events.end();
	     ++it) {
		if (id == it->ID()) {
			Events.erase(it);
			return 1;
		}
	}

	return 0;
}


int EventManagement::count() {
	return Events.size();
}

const string EventManagement::name(int id) {
	for (std::list<Event>::iterator it = Events.begin();
	     it != Events.end();
	     ++it) {
		if (id == it->ID()) {
			return it->name;
		}
	}

	return "";
}

const string EventManagement::cur_prog(int id) {
	for (std::list<Event>::iterator it = Events.begin();
	     it != Events.end();
	     ++it) {
		if (id == it->ID()) {
			if ( it->current_program_2_execute != it->end() ) {
				return *(it->current_program_2_execute);
			}
			else {
				return "";
			}
		}
	}

	return "";
}

int EventManagement::moveOn(int id) {
	for (std::list<Event>::iterator it = Events.begin();
	     it != Events.end();
	     ++it) {
		if (id == it->ID()) {
			it->current_program_2_execute++;
			if (it->current_program_2_execute == it->end() )
				return 0;

			return 1;
		}
	}

	return -1;
}

std::list<Event>::iterator EventManagement::event(int id) {
	for (std::list<Event>::iterator it = Events.begin();
	     it != Events.end();
	     ++it) {
		if (id == it->ID()) {
			return it;
		}
	}

	return NULL;
}

void EventManagement::adjustTimeouts(unsigned long sleep_time) {
	for (std::list<Event>::iterator it = Events.begin();
	     it != Events.end();
	     ++it) {
		it->increaseSleeptime(sleep_time);
	}
}

int EventManagement::eventIsRunning( const string eventName ) {
	for (std::list<Event>::iterator it = Events.begin();
	     it != Events.end();
	     ++it) {
		if ( eventName == it->name ) {
			return 1;
		}
	}

	return 0;
}
