#include "config.h"
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/stat.h>
#include <linux/major.h>
#include <linux/hdreg.h>

#include "powerlib.h"
#include "powerlib_local.h"
#include "string.h"

#define ACPI_LID_DIR "/proc/acpi/button/lid"


/*returns:
  NO_ACPI_ERROR
  NO_MODULE_ERROR -> processor module
  first cpu == 0 */
/* lower than zero or UNKNOWN == error*/

int getLidState(void){
	/* differ between several kernel implementations (where is the proc entry) here*/
 	char value[MAX_LINE_SIZE + 1] = "";
 	char temp[MAX_LINE_SIZE + 1] = "";
	char file[MAX_FILE_PATH + 1] = "";
	
	FILE *rdFile = NULL;
	int res;
	
	if(checkACPI() != ACPI)
		return NO_ACPI_ERROR;

	if (access(ACPI_LID_DIR, F_OK)){
		return NO_MODULE_ERROR;
	}
	if (getDirEntry(0, value, sizeof (value), ACPI_LID_DIR) < 0){
		pDebug(DBG_DIAG, "Could not determine file where ac status"
		       " is written in directory: %s!\n", ACPI_LID_DIR);
		goto ERROR;
	}
	/* possible to overwrite file ?!?*/
	res = snprintf(file, MAX_FILE_PATH, "%s/%s/state", ACPI_LID_DIR, value);
	if (res >= MAX_FILE_PATH) {
		pDebug (DBG_DIAG, "string too short, should be %d\n", res);
		goto ERROR;
	}
	pDebug(DBG_INFO, "Lid state file: %s\n", file);
	rdFile = fopen(file, "r");
	if (!rdFile){
		pDebug(DBG_DIAG, "Could not open file %s for reading out Lid state\n", file);
		goto ERROR;
	}
	
	if (getColonValue(rdFile, value, sizeof (value), temp, sizeof (temp)) < 0){
		pDebug(DBG_DIAG, "Syntax in file: %s changed! Or too less memory"
		       " for string allocated ! Please report this line"
		       " (including Function name and line) to trenn@suse.de!\n", file);
		goto ERROR;
	}
	else {
		pDebug(DBG_INFO, "Value: %s\n", value);
		fclose(rdFile);
		rdFile = NULL;
		if (!strcmp(value, "open"))
			return LID_OPEN;
		else if (!strcmp(value, "closed"))
			return LID_CLOSED;
		else{
			pDebug(DBG_DIAG, "Syntax in file: %s changed!"
			       " Please report this line (including Function name and line)"
			       " to trenn@suse.de!\n", ACPI_LID_DIR);
			goto ERROR;
		}
	}

 ERROR: 
	if(rdFile)
		fclose(rdFile);
	return UNKNOWN;
}
