#include "config.h"
#include <fcntl.h>
#include <sys/ioctl.h>
//#include <linux/types.h>
#include <dirent.h>

#include <unistd.h>
#include <stdlib.h>

#include "errno.h"

#include "powerlib_local.h"
#include "powerlib.h"

#define ACPI_FAN_DIR "/proc/acpi/fan"


/* returns x < 0 on failure */	
int getFanNum(){
	/* differ between several kernel implementations (where is the proc entry) here*/
	return getDevicesNum(ACPI_FAN_DIR);
}

/* returns: */
/*  0 if fan is on */
/*  1 if fan is off */
/*  -3 if fan device no. num does not exist */
/*  -4 if state file does not exist */
/*  -5 if state could not be figured out (probably changes in kernel syntax) */
/*  NO_ACPI_ERROR if no ACPI */
/*  NO_MODULE_ERROR if no fan directory (module probably not loaded) */
/*  NO_DEVICE_ERROR if there is a fan directory, but no device */

int getFanStatus(const int num){
	char file[MAX_FILE_PATH +1] = "";
	char value[MAX_LINE_SIZE + 1] = "";
	char def[MAX_LINE_SIZE + 1] = "";
	DIR *dir;
	struct dirent *dirent;
	FILE *rdFile;
	int dir_count = 0;
	int ret_val;

	if (checkACPI() != ACPI){
                pDebug(DBG_INFO, "no ACPI system => no fan control");
		return NO_ACPI_ERROR;
	}
	
	/* check whether module has been loaded and device is available */
	ret_val = check_ACPI_dir(ACPI_FAN_DIR);
	if (ret_val < 0)
		/* returns either NO_DEVICE_ERROR or NO_MODULE_ERROR */
		return ret_val;

	dir = opendir(ACPI_FAN_DIR);
	/* jump no of times in getting a new valid directory*/
	/* expect that dirent will always give back the same order of files(directories)*/
	while (1){
		dirent = readdir(dir);
		if (dirent <= 0){
			closedir(dir);
			return -3;
		}
		if (dirent->d_name[0] == '.')
			continue; /* skip dotfiles */
		if (dir_count == num)
			break;
		dir_count++;
	}
	snprintf(file, MAX_FILE_PATH, "%s/%s/state", ACPI_FAN_DIR, dirent->d_name);
	rdFile = fopen(file, "r");
	if (!rdFile)
		ret_val = -4;
	else{
                int ret = getColonValue(rdFile, value, sizeof (value), def, sizeof (def));
                fclose(rdFile);
		if (ret < 0)
			ret_val = -5;
		else {
			if (!memcmp(value, "on", 2))
				ret_val = 0;
			else if (!memcmp(value, "off", 3))
				ret_val = 1;
			else
				ret_val = -5;
		}
	}
	closedir(dir);
        return ret_val;
}
