#include "config.h"

#ifdef HAVE_LIBHAL

#include "hal.h"

#include <string.h>

DBusError dbus_error;
LibHalContext *hal_ctx = NULL;

int initHal() {
	dbus_error_init( &dbus_error );

	if ( hal_ctx != NULL ) {
		return 1;
	}

	DBusConnection *dbus_connection;

/*	DBusConnection *dbus_connection = dbus_bus_get( DBUS_BUS_SYSTEM,  &dbus_error  ); */

	/* we need a own connection for hal. Using the dbus connection
	 * from the daemon conflicts somehow when client requests come
	 * simultaneous with hal requests. Have to look deeper...(but not
	 * now)
	 * 
	 * hardcoded path to dbus socket for now. This has to change!
	 */
	dbus_connection = dbus_connection_open_private("unix:path=/var/run/dbus/system_bus_socket",
						       &dbus_error );
	if ( dbus_error_is_set( &dbus_error ) ) {
		pDebug(DBG_ERR, "could not set up connection to system bus: %s",
		       dbus_error.message );
		dbus_error_free( &dbus_error );
		return 0;
	}

	/* register our own connection */
	dbus_bus_register( dbus_connection, &dbus_error);

	if ( dbus_error_is_set( &dbus_error ) ) {
		pDebug(DBG_ERR, "could not set up connection to system bus: %s",
		       dbus_error.message );
		dbus_error_free( &dbus_error );
		return 0;
	}

	int hal_is_ready = dbus_bus_name_has_owner( dbus_connection,
					 "org.freedesktop.Hal",
					 &dbus_error );
	if ( ! hal_is_ready ) {
		pDebug(DBG_WARN, "hal not ready. We will try later..."
		       "This may sometimes happen during bootup");

		if ( dbus_error_is_set( &dbus_error ) ) {
			pDebug(DBG_ERR, "Error checking if hal service exists: %s",
			       dbus_error.message );
			dbus_error_free( &dbus_error );
		}
		return 0;
	}
	
	hal_ctx = libhal_ctx_new();

	/* setup dbus connection for hal */
	if ( ! libhal_ctx_set_dbus_connection( hal_ctx, dbus_connection ) ) {
		pDebug(DBG_ERR, "could not set up connection to dbus for hal");
		return 0;
	}

	/* init the hal library */
	if ( ! libhal_ctx_init( hal_ctx,  &dbus_error ) ) {
		pDebug(DBG_WARN, "could not init hal library: %s", dbus_error.message);
		dbus_error_free( &dbus_error );
		libhal_ctx_free( hal_ctx );
		hal_ctx = NULL;
		return 0;
	}


	return 1;
}

void freeHal() {
	libhal_ctx_free( hal_ctx );
	hal_ctx = NULL;
}

int hal_get_property_int( char* device,
			  char* property ) {

        if (!libhal_device_property_exists (hal_ctx, device, property, &dbus_error)) {
                pDebug(DBG_INFO, "property '%s' does not exist.", property);
                return -1;
        }
	
	int val = libhal_device_get_property_int(
		hal_ctx, device,
		property,  &dbus_error  );

	if ( dbus_error_is_set(  &dbus_error ) ) {
		pDebug(DBG_WARN, "Error fetching property '%s': '%s'",
			property, dbus_error.message );
		dbus_error_free(  &dbus_error  );
		return -1;
	}

	return val;
}


int hal_get_property_bool( char* device,
			   char* property ) {

        if (!libhal_device_property_exists (hal_ctx, device, property, &dbus_error)) {
                pDebug(DBG_INFO, "property '%s' does not exist.", property);
                return -1;
        }

	int val = libhal_device_get_property_bool(
		hal_ctx, device,
		property,  &dbus_error  );
	
	if ( dbus_error_is_set(  &dbus_error ) ) {
		pDebug(DBG_WARN, "Error fetching property '%s': '%s'",
			property, dbus_error.message );
		dbus_error_free(  &dbus_error  );
		return -1;
	}

	return val;
}


int hal_get_property_string( char* device,
			       char* property,
			       char *value ) {

        if (!libhal_device_property_exists (hal_ctx, device, property, &dbus_error)) {
                pDebug(DBG_INFO, "property '%s' does not exist.", property);
                return -1;
        }
	
	char* val = libhal_device_get_property_string(
		hal_ctx, device,
		property,  &dbus_error  );

	if ( dbus_error_is_set(  &dbus_error ) ) {
		pDebug(DBG_WARN, "Error fetching property '%s': '%s'",
			property, dbus_error.message );
		dbus_error_free(  &dbus_error  );
		return -1;
	}

	strcpy( value, val );

	libhal_free_string( val );

	return 1;
}
#endif /* HAVE_LIBHAL */
