#include "config.h"

#ifdef HAVE_LIBHAL

#ifndef POWERSAVE_HAL_H
#define POWERSAVE_HAL_H

#define DBUS_API_SUBJECT_TO_CHANGE 1
#include <hal/libhal.h>

#include "powerlib_local.h"

/** brief get integer property through hal
 *
 * fetches one interger value through hal. Throws a dbus error with error
 * message if needed.
 *
 * @param device the device from which should be fetched
 * @param property the property which should be fetched
 *
 * @return -1 on error
 */
int hal_get_property_int( char* device,
			  char* property );

/** brief get bool property through hal
 *
 * fetches one bool value through hal. Throws a dbus error with error
 * message if needed.
 *
 * @param device the device from which should be fetched
 * @param property the property which should be fetched
 *
 * @return -1 on error
 */
int hal_get_property_bool( char* device,
			  char* property );

/** brief get string property through hal
 *
 * fetches one string value through hal. Throws a dbus error with error
 * message if needed.
 *
 * @param device the device from which should be fetched
 * @param property the property which should be fetched
 * @param value a char pointer in which the fetched value is stored. 
 *        value must have enough memory.
 *
 * @return -1 on error
 */
int hal_get_property_string( char* device,
			     char* property,
			     char* value );

/** @brief inits the access to hal 
 *
 * if hal context is already initialized, does nothing.
 *
 * @return true on success, false on failure
*/
int initHal(void);

/** @brief frees the access to hal */
void freeHal(void);

#endif /* POWERSAVE_HAL_H */

#endif /* HAVE_LIBHAL */
