#ifndef POWERSAVE_POWERLIB_LOCAL_H
#define POWERSAVE_POWERLIB_LOCAL_H

/** @defgroup libpower_local local powerlib implementations
 * @ingroup libpower
 * @{
 */


#include <string.h>
#include <time.h>

#include "powerlib.h"

#ifndef max
#define max(a, b) (a > b) ? a : b;
#endif

/** @brief apm related header*/
typedef struct apm_info
{
    char driver_version[10];
    int apm_version_major;
    int apm_version_minor;
    int apm_flags;
    int ac_line_status;
    int battery_status;
    int battery_flags;
    int battery_percentage;
    int battery_time;
    int using_minutes;
}
apm_info;

/** @brief An array of this one is used to estimate the remaining 
   battery capacity in minutes.
*/
typedef struct capacity_snapshot
{
    time_t measure_point;
    int remaining_capacity;
}
capacity_snapshot;

/** @brief get amount of entries in a directory
 *
 * use this to determine available devices
 */
int getDevicesNum(const char* dir_name);

/** @brief get next value which is seperated by a colon
 *
 * e.g.: var:  56 mWh  returns 56 mWh in value and var in def
 */
//int getColonValue(FILE *file, char *value);
int getColonValue(FILE *file, char *value, const size_t value_size, char *def, const size_t def_size);

/** file: directory entry searched, e.g.: BAT0*/
/* give directory and position of the searched file*/
/* position is determined by just looping over the directory!*/
int getDirEntry(const int num, char *file, const size_t size_file, const char *dir_in);
/** @brief write a line to a file (e.g.: "3\n" to /acpi/sleep)*/
int _write_line(const char * filename, const char *fmt, ...);
/** @brief helper function to get general battery info for acpi*/
int get_ACPI_General_Battery_Info(BatteryGeneral *bg);
/** @brief helper function to get general battery info for apm*/
int get_APM_General_Battery_Info(BatteryGeneral *bd);
/** @brief reads out information from /proc/apm
 *
 * see apmd package and struct above apm_info
*/
int read_apm_info(apm_info * i);
/** @brief check whether module has been loaded and device is available */
int check_ACPI_dir(const char* dir);

#endif

/** @} */ // end of libpower_local group
