/** @defgroup client_exchange Handling client connections
 * @ingroup libpower
<h2>BASIC CONCEPT</h2>

<h3>Server ( the powersave daemon )</h3>

<ol>
  <li>provide dbus interface to which clients can connect
  <li>client sends connection request
  <li>Accept the connection
  <li>Receive the capabilities of the client
  <li>daemon checks if the client has sent valid capabilities

     <ul>
       <li>capabilities are not valid -> skip the client and send him a message about a
       failure 
   
       <li>capabilities are valid --> add the client (with the capabilities) to the list of
       connected clients 
     </ul>
  <li>Everytime there happens an event or a notification, broadcase
     this event/message over dbus 

</ol>

<h3>Client</h3>

<ol>
  <li>Send a signal for a connection with capabilities over the dbus interface
  <li>Set a match on the dbus interface for new events
  <li>Receive events and handle them
</ol>

*/


#ifndef POWERSAVE_CLIENTSOCKET_H
#define POWERSAVE_CLIENTSOCKET_H

#ifdef __cplusplus
extern "C" {
#endif


/** the maximum length of a dbus name (e.g. :1.3) */
#define DBUS_NAME_MAX 32

/** @defgroup client_capabilities Client modes: Messages clients can have
 * @ingroup client_exchange
 * Defines the different capabilities clients can request from the daemon.
 * Capabilities can be added up. These are simple integer values.
 * @{
 */
/** Forward notifications the daemon throuws */
#define CAPABILITY_NOTIFICATIONS 1
/** Forward screenlock requests */
#define CAPABILITY_SCREENLOCK 2
/** @} */ // end of client_modes group


/** @defgroup data_structures Data structures
 * @ingroup client_exchange
 * this node holds some values/states
 * @{
 */

/** 
 *\brief This node holds some values/states clients receive from the
 * daemon 
*/
typedef struct cNode {
	/** the clients name (dbus name)*/
	char name[DBUS_NAME_MAX];
	/** The capabilities to which the message belongs @ref client_capabilities */
	int capabilities;
} cNode;
/** @} */ // end of data structures group

	/** The provided functions are declared in the same order as they
	 * should be used when writing a client application */

#ifdef __cplusplus
}
#endif


#endif
