#include "config.h"
#include <unistd.h>
#include <strings.h>
#include <sys/time.h>
#include <sys/fcntl.h>
#include <sys/types.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <stdio.h>



#include "powerlib_local.h"
#include "powerlib.h"

#define ACPI_PROCESSOR_DIR "/proc/acpi/processor"
#define PROC_STAT_FILE "/proc/stat"
#define PROC_CPUINFO_FILE "/proc/cpuinfo"

/* returns:
   NO_ACPI_ERROR
   NO_MODULE_ERROR -> processor module
   -1 general error
*/
static unsigned long *ps_last_total_time = NULL, *ps_last_working_time = NULL;
static int cpuCount = -1;

int getThrottlingInfo(int *states, int *current ){
	char value[MAX_LINE_SIZE + 1] = "";
	char temp [MAX_LINE_SIZE + 1] = "";
	char file_path[MAX_FILE_PATH + 1] = "";
	char file[MAX_FILE_PATH + 1] = "";
	FILE *rdFile;
	int ret;
	
	*states=0;
	*current=0;
	if(checkACPI() != ACPI)
		return NO_ACPI_ERROR;
	
	if (access(ACPI_PROCESSOR_DIR, F_OK)){
		return NO_MODULE_ERROR;
	}
	/* expect that for every CPU the same number of throttling states exist*/
	getDirEntry(0, file, sizeof (file), ACPI_PROCESSOR_DIR);
	ret = snprintf(file_path, MAX_FILE_PATH, "%s/%s/throttling", ACPI_PROCESSOR_DIR, file);
	if (ret >= MAX_FILE_PATH){
		return -1;
	}
	rdFile = fopen(file_path, "r");
	if (!rdFile){
		return -1;
	}
	/* if we get out here, it is the <not supported> string*/
	if (getColonValue(rdFile, value, sizeof (value), temp, sizeof (temp)) < 0){
		fclose (rdFile);
		return -1;
	}
	else {
		*states = (int)strtol(value, NULL, 10);
	}
	if (getColonValue(rdFile, value, sizeof (value), temp, sizeof (temp)) < 0){
		pDebug(DBG_WARN, "Syntax in file: %s changed! Or too less memory"
		       " for string allocation. Please report this line"
		       " to trenn@suse.de.\n", file_path);
		ret = -1;
	}
	else {
		*current = (int)strtol(value+1, NULL, 10);
		/* test for error
			pDebug(DBG_DIAG, "Could not convert throttling value to integer!: %s\n", strerror(errno));
			fclose (rdFile);
			return -1;
		*/
		ret = 1;
	}
	fclose (rdFile);
	return ret;	
}

/* returns:
   NO_ACPI_ERROR
   NO_MODULE_ERROR -> processor module
   returns -2 if cpu does not exist
   -3 general error
*/

int getThrottlingInfoCPU(const int cpu, int *states, int *current){


	/*	char cpu_info_value[MAX_LINE_SIZE + 1];
	 */

	char value[MAX_LINE_SIZE + 1] = "";
	char temp [MAX_LINE_SIZE + 1] = "";
	char file_path[MAX_FILE_PATH + 1] = "";
	char file[MAX_FILE_PATH + 1] = "";
	FILE *rdFile;
	int ret;

	*states=0;
	*current=0;
	if(checkACPI() != ACPI)
		return NO_ACPI_ERROR;
	
	if (access(ACPI_PROCESSOR_DIR, F_OK)){
		return NO_MODULE_ERROR;
	}

	/* no hyperthreading check yet */
	if (getDirEntry(cpu, file, sizeof (file), ACPI_PROCESSOR_DIR) < 0)
		return -2;
		
	ret = snprintf(file_path, MAX_FILE_PATH, "%s/%s/throttling", ACPI_PROCESSOR_DIR, file);
	if (ret >= MAX_FILE_PATH){
		pDebug(DBG_DIAG, "File Path longer than MAX_FILE_PATH "
		       "for %s/%s/throttling\n", ACPI_PROCESSOR_DIR, file);
		ret = -3;
	}
	rdFile = fopen(file_path, "r");
	if (!rdFile){
		pDebug(DBG_DIAG, "Could not open file %s for reading"
		       " out AC information. err: %s\n", file_path, strerror(errno));
		ret = -3;
	}
	if (getColonValue(rdFile, value, sizeof (value), temp, sizeof (temp)) < 0){
		/* this is normal if there is the string <not supported> at this point*/
		pDebug(DBG_DIAG, "Throttling not supported\n");
		fclose (rdFile);
		return -3;
	}
	else {
		pDebug(DBG_INFO, "Throttling state: %s\n", value);
		*states = (int)strtol(value, NULL, 10);
	}
	if (getColonValue(rdFile, value, sizeof (value), temp, sizeof (temp)) < 0)
		ret = -3;
	else {
		pDebug(DBG_INFO, "Current Throttling state: %s\n", value);
		*current = (int)strtol(value+1, NULL, 10);
		ret = 1;
	}
	fclose (rdFile);
	return ret;	
}


/* returns:
   NO_ACPI_ERROR
   NO_MODULE_ERROR -> processor module
   -1 general error
*/

int setThrottlingState(const int state){
	int x;

	if(checkACPI() != ACPI)
		return NO_ACPI_ERROR;
	for (x = 0; setThrottlingStateCPU(x, state) > 0; x++);
	/* I do not check whether cpus above 0 (first cpu is zero) return error */
	if (x == 0)
		return NO_MODULE_ERROR;
	else
		return 1;
}
/*returns:
  NO_ACPI_ERROR
  NO_MODULE_ERROR -> processor module
  first cpu == 0 */
/* lower than zero or UNKNOWN == error*/

int setThrottlingStateCPU (const int cpu, const int state){
	char file[MAX_FILE_PATH + 1 ] = "";
	char file_path[MAX_FILE_PATH + 1] = "";
	int ret = 0;

	if(checkACPI() != ACPI)
		return NO_ACPI_ERROR;
	if (getDirEntry(cpu, file, sizeof (file), ACPI_PROCESSOR_DIR) < 0)
		return NO_MODULE_ERROR;

	ret = snprintf(file_path, MAX_FILE_PATH, "%s/%s/throttling",
		       ACPI_PROCESSOR_DIR, file);
	if (ret >= MAX_FILE_PATH){
		pDebug(DBG_DIAG, "File Path longer than MAX_FILE_PATH"
		       " for %s/%s/throttling\n", ACPI_PROCESSOR_DIR, file);
		return -1;
	}
	if (_write_line(file_path, "%d", state) < 0){
			pDebug(DBG_DIAG, "Could not set throttling state to %d in file %s\n", 
			       state, file_path);
			return -1;
	}
	pDebug (DBG_INFO, "cpu %d set to throttling state %d\n", cpu, state );
	return 1;
}

/*returns:
  first cpu == 0 */
/* lower than zero or UNKNOWN == error*/

int setThrottlingPercentCPU(const int cpu, const int percent){
	int states, current, requested;

	if (percent < 0 || percent > 100){
		return -1;
	}
	requested = getThrottlingInfoCPU(cpu, &states, &current);
	if (requested < 0)
		return requested;
	requested = (int)roundf((percent * states) / 100);
	if (setThrottlingStateCPU(cpu, requested) < 0){
		pDebug(DBG_DIAG, "Could not set throttling state on cpu %d"
		       "to state %d - %d percent\n", cpu, requested, percent);
		return -1;
	}
	return 1;
}

int setThrottlingPercent(const int percent){
	int x;
	int cpus = 0;
	if ((cpus = getDevicesNum(ACPI_PROCESSOR_DIR)) < 0){
		return -1;
	}
	
	for (x=0; x < cpus;x++){
		if (setThrottlingPercentCPU(x, percent) < 0)
			return -1;
	}
	return 1;
}

/* be careful this method waits a second ...*/
/* Function only available on i686 and x86_64 machines!*/

/* returns -1.0 if sleep is interrupted, result will be wrong */
/* returns -2.0 if compiled for a not supported architecture! */
/* returns -3.0 if /proc/cpuinfo is not accessable and supported 
   for architecture could not be evaluated! */

float getRealProcessorSpeed() {

#if defined(__i386__) || defined(__x86_64__)
	unsigned long long int x;
	//	struct timezone tz;
        struct timeval tvstart, tvstop;
        u_int64_t cycles[2];
	unsigned int microseconds; /* total time taken */
	int sleep_left, cpuinfo_fd, tsc_support;
	float ret;
	char buffer[MAX_LINE_SIZE +1] = "";
	tsc_support = 0;

	/* search cpuinfo for tsc register support, if we find it, 
	   the calculation is supported for this machine */
	cpuinfo_fd = open("/proc/cpuinfo", O_RDONLY);
	if (cpuinfo_fd < 0)
		return -3.0;
	while (read(cpuinfo_fd, buffer, MAX_LINE_SIZE) > 0){
		if (strstr(buffer, "tsc")){
			tsc_support = 1;
			break;
		}
	}
	close (cpuinfo_fd);

	if (tsc_support != 1)
		return -2.0;
	
	/* get this function in cached memory */
	gettimeofday(&tvstart, NULL);
	__asm__ volatile ("rdtsc" : "=A" (x));
	cycles[0] = x;
	gettimeofday(&tvstart, NULL);

	/* we don't trust that this is any specific length of time */
	sleep_left = sleep(1);
	if (sleep_left || errno == EINTR)
		return -1.0;
	__asm__ volatile ("rdtsc" : "=A" (x));
	cycles[1] = x;
	gettimeofday(&tvstop, NULL);
	microseconds = ((tvstop.tv_sec-tvstart.tv_sec)*1000000) +
		(tvstop.tv_usec-tvstart.tv_usec);

	ret = (float)(cycles[1]-cycles[0])/microseconds;
	pDebug(DBG_DIAG, "%f MHz processor.\n", ret);
	return ret;

#else
	// not implemented
	return -2.0;
#endif
}

/* This function exists for compatibility */
double getCPULoad(){
	return getCPULoadConsiderNice(0);
}
/* returns "Idleness" of the processor in percent*/
/* since the last time this function has been called*/

/* returns -2 if /proc/stat has not been updated yet*/

/* on error -1 is returned*/

/* if you poll idleness of processor, ignore the result */
/* of the first time you invoke this function, because*/
/* it will be the total idleness of cpu since the machine*/
/* has been powered on*/
double getCPULoadConsiderNice(int consider_nice){
	unsigned long total_elapsed, working_elapsed;
	char what[32] = "";
	unsigned long user_time, nice_time, system_time, idle_time, total_time, iowait_time;
	static unsigned long last_total_time = 0, last_working_time = 0;
	unsigned scan_ret;
	char line[256] = "";
	FILE *fp;

	/*	pDebug(DBG_INFO, "Before calculation: last_total_time: %d, last_idle_time: %d\n",*/
	/*	last_total_time, last_working_time);*/

	if ((fp = fopen(PROC_STAT_FILE, "r")) == NULL){
		/* let errno be set for higher applications...*/
		pDebug(DBG_DIAG, "Could not open file: %s; Error: %s",
		       PROC_STAT_FILE, strerror(errno));
		return -1;
	}

	/* search for cpu line ...*/
	/* is the "cpu " string always at the beginning? -> Expect yes, but had*/
	/* no look in the kernel*/

	/* what about multi processor systems? -> not supported at the moment*/
	while (fgets(line, 255, fp) != NULL){
		if (!memcmp(line, "cpu ", 4))
			break;
	}
	if (memcmp(line, "cpu ", 4)){
		pDebug(DBG_DIAG, "Could not find cpu string in %s\n", PROC_STAT_FILE);
		fclose(fp);
		return -1;
	}
	iowait_time=0; /* initialized, since it is simply not there in 2.4 */
	scan_ret = sscanf(line, "%s %lu %lu %lu %lu %lu", what, &user_time, &nice_time,
	       &system_time, &idle_time, &iowait_time);
	fclose(fp);
	if (scan_ret < 5){
		pDebug (DBG_DIAG, "Only %d values converted in sscanf, "
			"syntax seems to have changed in %s\n", scan_ret, PROC_STAT_FILE);
		return -1;
	}
	/*	pDebug(DBG_DIAG, "%s, %lu, %lu, %lu, %lu\n",*/
	/*	       what, user_time, nice_time, system_time, idle_time);*/

	/* count nice time and i/o-wait as idle time*/
	if (consider_nice){
		idle_time += iowait_time;
		total_time = nice_time + user_time + system_time + idle_time;
	}
	else{
		idle_time += (nice_time + iowait_time);
		total_time = user_time + system_time + idle_time;
	}
	total_elapsed = total_time - last_total_time;
	last_total_time = total_time;
	if (consider_nice){
		working_elapsed = (user_time + system_time + nice_time) - last_working_time;
		last_working_time = user_time + system_time + nice_time;
	}
	else{
		working_elapsed = (user_time + system_time) - last_working_time;
		last_working_time = user_time + system_time;
	}
	if (!total_elapsed){
		pDebug(DBG_DIAG, "Could not evalutate CPU load, %s is not updated,"
		       " yet, please slow down polling this file!\n", PROC_STAT_FILE);
		return -2;
	}

	if (total_time > 0)
		return (double)working_elapsed / (double)total_elapsed * 100.0;
	else
		return -1;
}

double getCPULoadSMP(const int consider_nice, const int cpu){

	unsigned long total_elapsed, working_elapsed;
	char what[32];
	unsigned long user_time, nice_time, system_time, idle_time, total_time, iowait_time;
	unsigned scan_ret;
	char line[256];
	char cpuString[7];
	FILE *fp;

	if (cpu < 0 || cpu > MAX_SUPPORTED_CPUS){
		errno = EINVAL;
		return -10.0;
	}

	if (cpuCount == -1){
		cpuCount = getCPUCount();
		if (cpuCount <= 0){
			errno = ENODEV;
			return -20;
		}
		ps_last_total_time = (unsigned long*) calloc(cpuCount, sizeof(unsigned long));
       		ps_last_working_time = (unsigned long*) calloc(cpuCount, sizeof(unsigned long));
		/*		for (x=0; x < cpuCount; x++){
			ps_last_total_time[x] = (unsigned long*) malloc(sizeof(unsigned long));
			ps_last_working_time[x] = (unsigned long*) malloc(sizeof(unsigned long));
			*ps_last_total_time[x] = 0;
			*ps_last_working_time[x] = 0;
		}
		*/
	}

	/* just to be sure we get no segfault. Should I check array as well? 
	   Do I need a check at all?*/
	if (ps_last_total_time == NULL || ps_last_working_time == NULL){
		errno = EFAULT;
		return -40;
	}

	if (cpu > cpuCount){
		errno = ENODEV;
		return -30;
	}

	sprintf(cpuString, "cpu%d", cpu);

	/*	pDebug(DBG_INFO, "Before calculation: last_total_time: %d, last_idle_time: %d\n",*/
	/*	last_total_time, last_working_time);*/

	if ((fp = fopen(PROC_STAT_FILE, "r")) == NULL){
		/* let errno be set for higher applications...*/
		pDebug(DBG_DIAG, "Could not open file: %s; Error: %s",
		       PROC_STAT_FILE, strerror(errno));
		return -1;
	}

	/* search for cpu line ...*/
	/* is the "cpu " string always at the beginning? -> Expect yes, but had*/
	/* no look in the kernel*/

	/* what about multi processor systems? -> not supported at the moment*/
	while (fgets(line, 255, fp) != NULL){
		if (!memcmp(line, cpuString, 4))
			break;
	}
	if (memcmp(line, cpuString, 4)){
		pDebug(DBG_DIAG, "Could not find cpu string in %s in file %s\n", cpuString, PROC_STAT_FILE);
		fclose(fp);
		return -1;
	}
	iowait_time=0; /* initialized, since it is simply not there in 2.4 */
	scan_ret = sscanf(line, "%s %lu %lu %lu %lu %lu", what, &user_time, &nice_time,
	       &system_time, &idle_time, &iowait_time);
	fclose(fp);
	if (scan_ret < 5){
		pDebug (DBG_DIAG, "Only %d values converted in sscanf, "
			"syntax seems to have changed in %s\n", scan_ret, PROC_STAT_FILE);
		return -1;
	}

	if (consider_nice){
		idle_time += iowait_time;
		total_time = nice_time + user_time + system_time + idle_time;
	}
	else{
		idle_time += (nice_time + iowait_time);
		total_time = user_time + system_time + idle_time;
	}
	total_elapsed = total_time - ps_last_total_time[cpu];
	ps_last_total_time[cpu] = total_time;
	if (consider_nice){
		working_elapsed = (user_time + system_time + nice_time) - ps_last_working_time[cpu];
		ps_last_working_time[cpu] = user_time + system_time + nice_time;
	}
	else{
		working_elapsed = (user_time + system_time) - ps_last_working_time[cpu];
		ps_last_working_time[cpu] = user_time + system_time;
	}
	if (!total_elapsed){
		pDebug(DBG_DIAG, "Could not evalutate CPU load, %s is not updated, yet,"
		       " please slow down polling this file!\n", PROC_STAT_FILE);
		return -2;
	}

	if (total_time > 0)
		return (double)working_elapsed / (double)total_elapsed * 100.0;
	else
		return -1;
}


int getCPUCount(){

	FILE *fp;
	char line[256];
	int count = 0;

	if ((fp = fopen(PROC_CPUINFO_FILE, "r")) == NULL){
		/* let errno be set for higher applications...*/
		pDebug(DBG_DIAG, "Could not open file: %s; Error: %s",
		       PROC_CPUINFO_FILE, strerror(errno));
		return -1;
	}

	/* search how many lines start with processor in /proc/cpuinfo hope 
	   that's save to evaluate amount of CPUs */
	while (fgets(line, 255, fp) != NULL){
		if (!memcmp(line, "processor", 9))
			count++;
	}
	fclose(fp);
	if (count > 0)
		return count;
	else{
		pDebug(DBG_ERR, "No processor found");
		return -2;
	}
}

/* needs to be invoked, once the getCPULoadSMP function is called 
 to free memory. The getCPULoadSMP may be called multiple times, when
 not using the func anymore call this one */
void freeCPULoadSMPCalls(){
	if (cpuCount != -1){
		free (ps_last_working_time);
		free (ps_last_total_time);
	}
}
